using System;
using System.Collections.ObjectModel;
using System.Reflection;
using System.Text.RegularExpressions;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;

namespace ThinkGeo.MapSuite.Layers
{
    [Serializable]
    public class LegendItem
    {
        public LegendItem()
        {
            Width = 160;
            Height = 25;
            ImageWidth = 16;
            ImageHeight = 16;
            ImageTopPadding = 5;
            ImageBottomPadding = 5;
            ImageLeftPadding = 10;
            ImageRightPadding = 5;
            TextTopPadding = 5;
            TextLeftPadding = 10;
        }

        public LegendItem(int width, int height, float imageWidth, float imageHeight, Style imageStyle, TextStyle textStyle)
        {
            this.width = width;
            this.height = height;
            this.imageWidth = imageWidth;
            this.imageHeight = imageHeight;
            this.imageStyle = imageStyle;
            this.textStyle = textStyle;
        }

        [Obfuscation(Exclude = true)]
        private float width;

        [Obfuscation(Exclude = true)]
        private float height;

        [Obfuscation(Exclude = true)]
        private float imageWidth;

        [Obfuscation(Exclude = true)]
        private float imageHeight;

        [Obfuscation(Exclude = true)]
        private float topPadding;

        [Obfuscation(Exclude = true)]
        private float bottomPadding;

        [Obfuscation(Exclude = true)]
        private float leftPadding;

        [Obfuscation(Exclude = true)]
        private float rightPadding;

        [Obfuscation(Exclude = true)]
        private float imageTopPadding;

        [Obfuscation(Exclude = true)]
        private float imageBottomPadding;

        [Obfuscation(Exclude = true)]
        private float imageLeftPadding;

        [Obfuscation(Exclude = true)]
        private float imageRightPadding;

        [Obfuscation(Exclude = true)]
        private float textTopPadding;

        [Obfuscation(Exclude = true)]
        private float textBottomPadding;

        [Obfuscation(Exclude = true)]
        private float textLeftPadding;

        [Obfuscation(Exclude = true)]
        private float textRightPadding;

        [Obfuscation(Exclude = true)]
        private Style imageStyle;

        [Obfuscation(Exclude = true)]
        private TextStyle textStyle;

        [Obfuscation(Exclude = true)]
        private AreaStyle backgroundMask;

        [Obfuscation(Exclude = true)]
        private AreaStyle textMask;

        [Obfuscation(Exclude = true)]
        private AreaStyle imageMask;

        [Obfuscation(Exclude = true)]
        private LegendImageJustificationMode imageJustificationMode;

        [Obfuscation(Exclude = true)]
        private bool drawn;

        [Obfuscation(Exclude = true)]
        private float originalSize;

        [Obfuscation(Exclude = true)]
        private float ratio = 1;

        public float TopPadding
        {
            get { return topPadding; }
            set { topPadding = value; }
        }

        public float BottomPadding
        {
            get { return bottomPadding; }
            set { bottomPadding = value; }
        }

        public float LeftPadding
        {
            get { return leftPadding; }
            set { leftPadding = value; }
        }

        public float RightPadding
        {
            get { return rightPadding; }
            set { rightPadding = value; }
        }

        public float ImageTopPadding
        {
            get { return imageTopPadding; }
            set { imageTopPadding = value; }
        }

        public float ImageBottomPadding
        {
            get { return imageBottomPadding; }
            set { imageBottomPadding = value; }
        }

        public float ImageLeftPadding
        {
            get { return imageLeftPadding; }
            set { imageLeftPadding = value; }
        }

        public float ImageRightPadding
        {
            get { return imageRightPadding; }
            set { imageRightPadding = value; }
        }

        public float TextTopPadding
        {
            get { return textTopPadding; }
            set { textTopPadding = value; }
        }

        public float TextBottomPadding
        {
            get { return textBottomPadding; }
            set { textBottomPadding = value; }
        }

        public float TextLeftPadding
        {
            get { return textLeftPadding; }
            set { textLeftPadding = value; }
        }

        public float TextRightPadding
        {
            get { return textRightPadding; }
            set { textRightPadding = value; }
        }

        public float Width
        {
            get { return width; }
            set { width = value; }
        }

        public float Height
        {
            get { return height; }
            set { height = value; }
        }

        public float ImageWidth
        {
            get { return imageWidth; }
            set { imageWidth = value; }
        }

        public float ImageHeight
        {
            get { return imageHeight; }
            set { imageHeight = value; }
        }

        public Style ImageStyle
        {
            get { return imageStyle; }
            set { imageStyle = value; }
        }

        public TextStyle TextStyle
        {
            get { return textStyle; }
            set { textStyle = value; }
        }

        public AreaStyle BackgroundMask
        {
            get { return backgroundMask; }
            set { backgroundMask = value; }
        }

        public AreaStyle TextMask
        {
            get { return textMask; }
            set { textMask = value; }
        }

        public AreaStyle ImageMask
        {
            get { return imageMask; }
            set { imageMask = value; }
        }

        public float Ratio
        {
            get { return ratio; }
            set { ratio = value; }
        }

        public LegendImageJustificationMode ImageJustificationMode
        {
            get { return imageJustificationMode; }
            set { imageJustificationMode = value; }
        }

        public void Draw(GeoCanvas adornmentGeoCanvas, Collection<SimpleCandidate> labelsInAllLayers, LegendDrawingParameters legendDrawingParameters)
        {
            ValidatorHelper.CheckInputValueIsLargerThanZero(width, "width");
            ValidatorHelper.CheckInputValueIsLargerThanZero(height, "height");
            ValidatorHelper.CheckGeoCanvasIsInDrawing(adornmentGeoCanvas.IsDrawing);
            ValidatorHelper.CheckObjectIsNotNull(labelsInAllLayers, "labeledInLayers");

            DrawCore(adornmentGeoCanvas, labelsInAllLayers, legendDrawingParameters);
        }

        protected virtual void DrawCore(GeoCanvas adornmentGeoCanvas, Collection<SimpleCandidate> labelsInAllLayers, LegendDrawingParameters legendDrawingParameters)
        {
            float xOffset = legendDrawingParameters.XOffset;
            float yOffset = legendDrawingParameters.YOffset;
            float totalImageLeftPadding = 0;
            float totalTextLeftPadding = 0;
            float totalImageTopPadding = yOffset + TopPadding + ImageTopPadding;
            float totalTextTopPadding = yOffset + TopPadding + TextTopPadding;

            DrawingRectangleF textLocation = new DrawingRectangleF();
            if (TextStyle != null && !string.IsNullOrEmpty(TextStyle.TextColumnName))
            {
                int warplength = CustomWarpLength();
                string text = ChangeTheTextLengthByMaxTextLength(warplength);
                if (TextStyle.Font != null)
                {
                    if (drawn)
                    {
                        TextStyle.Font = new GeoFont(TextStyle.Font.FontName, TextStyle.Font.Size * ratio, TextStyle.Font.Style);
                    }
                }
                else
                {
                    TextStyle.Font = new GeoFont("Arial", adornmentGeoCanvas.Width / 19.2f);
                }
                textLocation = adornmentGeoCanvas.MeasureText(text, TextStyle.Font);
            }

            switch (ImageJustificationMode)
            {
                case LegendImageJustificationMode.JustifyImageRight:
                    totalImageLeftPadding = xOffset + LeftPadding + TextLeftPadding + textLocation.Width + TextRightPadding + ImageLeftPadding;
                    totalTextLeftPadding = xOffset + LeftPadding + TextLeftPadding;
                    break;

                default:
                    totalImageLeftPadding = xOffset + LeftPadding * ratio + ImageLeftPadding * ratio;
                    totalTextLeftPadding = xOffset + LeftPadding * ratio + ImageLeftPadding * ratio + ImageWidth * ratio + ImageRightPadding * ratio + TextLeftPadding * ratio;
                    break;
            }

            if (BackgroundMask != null)
            {
                float newWidth = ratio * ImageWidth + textLocation.Width + totalTextLeftPadding + totalImageLeftPadding;
                float newHeight = height * ratio;
                DrawingRectangleF backgroudExtent = new DrawingRectangleF(newWidth / 2 + xOffset + LeftPadding, newHeight / 2 + yOffset * ratio + TopPadding, newWidth, newHeight);
                BackgroundMask.DrawSample(adornmentGeoCanvas, backgroudExtent);
            }
            if (ImageStyle != null)
            {
                float newImageWidth = ratio * ImageWidth;
                float newImageHeight = newImageWidth;

                DrawingRectangleF imageExtent = new DrawingRectangleF(newImageWidth / 2 + totalImageLeftPadding, newImageHeight / 2 + totalImageTopPadding * ratio, newImageWidth, newImageHeight);
                if (ImageMask != null)
                {
                    ImageMask.DrawSample(adornmentGeoCanvas, imageExtent);
                }
                if (imageStyle is PointStyle)
                {
                    (imageStyle as PointStyle).ImageScale = ratio;
                }
                ImageStyle.DrawSample(adornmentGeoCanvas, imageExtent);
            }
            if (TextStyle != null && !string.IsNullOrEmpty(TextStyle.TextColumnName))
            {
                DrawingRectangleF textExtent = new DrawingRectangleF(textLocation.CenterX + totalTextLeftPadding, textLocation.CenterY + totalTextTopPadding * ratio, textLocation.Width, textLocation.Height);
                if (TextMask != null) { TextMask.DrawSample(adornmentGeoCanvas, textExtent); }
                TextStyle.DrawSample(adornmentGeoCanvas, textExtent);

                if (!drawn)
                {
                    originalSize = TextStyle.Font.Size;
                }
                drawn = true;
            }
        }

        protected virtual int CustomWarpLength()
        {
	    // This API get changed in MapSuite package, so we need to add comment here to update this package, or else the latest beta version will throw "method not found" exception
            GeoCanvas textGeoCanvas = GeoCanvas.CreatePlatformGeoCanvas();
            int textWidth = (int)textGeoCanvas.MeasureText(TextStyle.TextColumnName, TextStyle.Font).Width;
            int warplength = 0;

            float textTotalWidth = Width - (ImageWidth + ImageLeftPadding + ImageRightPadding + TextLeftPadding + TextRightPadding);

            if (textTotalWidth < textWidth)
            {
            }

            return warplength;
        }

        private string ChangeTheTextLengthByMaxTextLength(int wordWrapLength)
        {
            string tempText = textStyle.TextColumnName;
            if (wordWrapLength != 0)
            {
                string cuttedText = tempText;
                string cuttingText = "";
                while (cuttedText.Length > wordWrapLength)
                {
                    if (cuttedText.Contains(" "))
                    {
                        cuttedText = Regex.Replace(cuttedText, @"\s+", " ");
                        if (cuttedText.IndexOf(" ", 0, wordWrapLength) + cuttedText.IndexOf(" ", wordWrapLength) < wordWrapLength * 2)
                        {
                            cuttingText = cuttingText + cuttedText.Substring(0, cuttedText.IndexOf(" ")) + "\r\n";
                            cuttedText = cuttedText.Substring(cuttedText.IndexOf(" ") + 1);
                        }
                        else
                        {
                            cuttingText = cuttingText + cuttedText.Substring(0, cuttedText.IndexOf(" ", wordWrapLength)) + "\r\n";
                            cuttedText = cuttedText.Substring(cuttedText.IndexOf(" ", wordWrapLength) + 1);
                        }
                    }
                    else
                        break;
                }
                tempText = cuttingText + cuttedText;
            }
            return tempText;
        }
    }
}